.cseg
; ********************************************************************************************
; DISP-Unterprogramme V1.0 für DOGM128-Serie mit RAM-Schattenbuffer
;
; (c) 2020 by dc7gb@vfdb.org
;
; lizenziert unter: https://creativecommons.org/licenses/by-nc-sa/4.0/
; ********************************************************************************************
;
; Das DOGM-Display wird über VPORT3 angesprochen. Dieser Port kann in der Inititialisierungsphase
; des Programms über  PORTCFG_VPCTRLn  auf jeden beliebigen physikalischen PORT eingestellt werden.
; Das Display wird über diese Leitungen in VPORT3 angesteuert:
;
;	LCD_RST		RESET-Pin des Displays
;	LCD_A0		Address-Select
;	LCD_CS		Display-Select
;	LCD_SI		Display-Dateneingang
;	LCD_SCL		Display-Takteingang
;
; Das Disply wird in "reverse-position" betrieben (Pinreihe 21..40 liegt unten). Der Koordina-
; tenursprung (X=0, Y=0) aller Grafikoperationen liegt oben links! Alle Operationen finden
; in einem Schattenbuffer statt. In (dispwp) steht der Zeiger zum Anfang des Bildspeicher-RAM
; (1024 Bytes). Es können mehrere virtuelle Bildspeicher aufgebaut und verwaltet werden.
;
; Das im Bildspeicher stehende Muster muss zur Anzeige mit  DISP_REFRESH  ins Display übertragen
; werden. Das RAM wird dabei für den nächsten Bildaufbau gelöscht (auf $00 gesetzt). Es ist nicht
; sinnvoll diese Ausgabe öfter als etwa 10x pro Sekunde zu machen, da das LCD schnellere Änderungen
; kaum anzeigen kann!
;
; Zeichenroutinen arbeiten mit positiven Offset vom Ursprung in (dispx) und (dispy). Negative
; Längen und Breiten sind nicht möglich. Es findet kein Plausibilitätscheck der übergebenben
; Paramter statt! Die folgenden globalen Variablen beeinflussen die Text- und/oder Grafikausgabe:
;
;	dispx		X-Position (0..127) der Ausgabe
;	dispy		Y-Position (0..63) der Ausgabe
;	dispr			(hier nicht vorhanden)
;	disppage	Y-Position (0..7) für elementare Routinen
;	dispbreite	Breite (0..127) einer Grafikausgabe (Ende: dispx + dispbreite)
;	disphoehe	Höhe (0..63) einer Grafikausgabe (Ende: dixpy + disphoehe)
;	dispmuster	zu verwendendes Schreibmuster bei Grafikausgaben
;	dispeor		bitweise Invertierung der 1-Bits im Bildspeicher bei Grafikausgaben
;	dispmaske	zu schützende Bits (0) bei Grafikausgaben
;	dispshift	Y-Feinverschiebung
;	dispfont	Fontnummer (hier: 1 oder 2) für ASCII-Ausgaben
;
; Zur ASCII-Ausgabe stehen zwei Fonts (1: F4x6, 2: F6x8) zur Verfügung. Ab $80 sind Umlaute
; und einige spezielle Grafiksymbole bis $92 in beiden Fonts identisch definiert. Ab $93 
; unterscheiden sich die Definitionen in den beiden Fonts. Alle Ausgaben können zerstörend
; oder überlagernd auf das vorhandene Bildmuster erfolgen. Die Positionierung ist pixelgenau
; möglich. Dazu kann ein EXOR-Muster (1-Bit) in  dispeor  angegeben werden. In  dispmaske
; können 0-Bits angegeben werden, die bei der Ausgabe nicht verändert werden dürfen.
;
; Innerhalb eines Strings können X-Feinpositionierungen angegeben werden. Sie haben die Codierung:
; 0b1111xxxx, wobei xxxx eine vorzeichenbehaftete Zahl (-8...+7) darstellt. Die Y-Feinpositionierung
; erfolgt mit 0b1110xxxx und vorzeichenbehafteten Offset xxxx = (-8..+7). X- und Y-Feinoffset gelten
; nur in der aktuellen Zeile. 
;
; Die Feinpositionierung innerhalb eines Strings gilt nur für die aktuelle Schreibzeile. Das
; Steuerzeichen CR setzt die X-Koordinate wieder auf vorher den übergebenen Wert in (dispx). Mit LF
; wird die Y-Koorddinate in (dispy) um die eingestellte Fonthöhe erhöht (nach unten) verschoben.
;
; Die Routinen verwenden folgende CPU-Register:
;
;	akku0	= r0	Rechenregister
;	akku1	= r1	Rechenregister
;	akku2		Rechen- und Zählregister
;	akku3		Musterregister
;	const		Konstantenregister
;	rest		Divisionsrest
;	temp	> r15	allgemeines Datenregister
;	buffer	> r15	allgemeines Datenregister
;	counter	> r15	Zählregister
;	YH:YL	r28:29	16 Bit Zeiger ins RAM
;	ZH:ZL	r20:31	16 Bit Zeiger ins ROM
;
; Routinen:
; ---------
;
;	DISP_REFRESH	überträgt den Bildspeicher ab (dispwp) an das LCD und löscht den Bildspeicherinhalt
;	DISP_CIRCLE		(hier nicht vorhanden)
;	DISP_RAHMEN	zeichnet Rahmen ab (dispx), (dispy) mit (disphoehe), (dispbreite)
;	DISP_RECT	zeichnet Rechteck ab (dispx), (dispy) mit (disphoehe), (dispbreite), gefüllt mit (temp)
;	DISP_DOT	setzt einen Punkt an (dispx), (dispy)
;	DISP_LINE		(hier nicht vorhanden)
;	DISP_HLINE	zeichnet horizontale Linie ab (dispx), (dispy) mit Breite (dispbreite) aus 
;	DISP_VLINE	zeichnet vertikale Linie ab (dispx), (dispy) mit Länge (disphoehe) 
;	DISP_DEZ2R	gibt (akku0) mit Vorzeichen. 2-stellig, dezimal, rechts-zentriert aus
;	DISP_CRLF	erhöht (dispx) um Fontbreite und dispy) um Fonthöhe
;	DISP_SPACE	erhöht (dispx) um Fontbreite
;	DISP_PRINTROM	gibt String ab (Z+) im ROM bis Nullbyte oder mit Begrenzung in (counter) aus
;	DISP_PRINTRAM	gibt String ab (Y+) im RAM bis Nullbyte oder mit Begrenzung in (counter) aus 
;	DISP_HEXSP	gibt Byte in (temp) in HEX ab (dispx), (dispy) + SPACE aus
;	DISP_HEX	gibt Byte in (temp) in HEX ab (dispx), (dispy) aus
;	DISP_NIBBLE	gibt Halb-Byte in (temp) in HEX ab (dispx), (dispy) aus
;	DISP_OUT	gibt ASCII-Zeichen in (temp) ab (dispx), (dispy) ggf über 2 Pages aus
;	DISP_PAGEOUT	gibt ASCII-Zeichen in (temp) ab (dispx) in (disppage) aus
;	DISP_SETPAGE	setzt  die Page in (disppage) im Display-RAM
;	DISP_SETZPAGEB	setzt die Page in (buffer) im Display-RAM
;	DISP_CLRPAGE	löscht die Page in (disppage) im Display-RAM mit Muster
;	DISP_CLRPAGEB	löscht die Page in (buffer) im Display-RAM mit Muster
;	DISP_FILLPAGE	füllt die Page in (disppage) im Display-RAM mit Muster in (temp)
;	DISP_FILLPAGEB	füllt die Page in (buffer) im Display-RAM mit Muster in (temp)
;	DISP_SETRAM	setzt das ganze Display-RAM
;	DISP_CLRRAM	löscht das ganze Display-RAM
;	DISP_FILLRAM	füllt das ganze Display-RAM mit Muster in (temp)
;
; Basisroutinen:
; --------------
;
;	DISP_FONT1	aktiviert den kleinen (4x6)-Font zur textausgabe
;	DISP_FONT2	aktiviert den normalen (6x8)-Font zur Textausgabe
;	DISP_PAGESET	(buffer) setzt die aktive Zeile (0..7) im LCD 
;	DISP_COLUMNSET	(buffer) setzt die aktive Spalte (0..127) im LCD
;	DISP_CMD	(temp) ins LCD-Kommandoregister schreiben
;	DISP_WRITE	(temp) ins LCD-Datenregister schreiben
;	DISP_INIT	DISP-Variablen und LCD-Dispaly initialisieren
;
; Serviceroutinen:
; ----------------
;
;	DIV8CONST10	teile (akku0) durch 10, Quotient in (akku0) und Teilungsrest in (rest)
;	DIV8		teile (akku0) durch (const), Quotient in (akku0) und Teilungsrest in (rest)
;
; ********************************************************************************************


; -----------------------------------------------------------------------------------------------
; Display-RAM zum LCD-Display übertragen, Bildspeicher löschen:		[3,29ms, 105266 cyc]
; -----------------------------------------------------------------------------------------------

DISP_REFRESH:	ldsw	Z,dispwp		; Display-RAM
		ldiw	Y,1024			; Größe des Buffers
		ldi	counter,0		; Display
		mov	buffer,counter		; ...von oben
		rcall	disp_pageset		; ...beschreiben
		ldi	buffer,4		; Zeile von links
		rcall	disp_columnset		; ...beginnen
		clr	const			; 0-Byte zum Löschen des RAMs
;
DISP_REFLOOP:	ld	temp,Z			; 8 Pixel laden
		st	Z+,const		; ...RAM neu initialisieren
		rcall	disp_write		; ...und Element ausgeben
		inc	buffer			; Zeilenposition zählen
		cpi	buffer,(4+128)		; Zeilenende erreicht?
		brne	disp_refend		; nein...
		inc	counter			; Page erhöhen
		mov	buffer,counter		; neue Page
		rcall	disp_pageset		; ...setzen
		ldi	buffer,4		; Zeilenadresse auf Anfang
		rcall	disp_columnset		; ...setzen
DISP_REFEND:	sbiw	Y,1			; Ausgabe zählen
		brne	disp_refloop		; nein...
; ----------------------------------------------
		ret


; -----------------------------------------------------------------------------------------------
; Rahmen zeichnen von (dispx, dispy) mit (dispbreite, disphoehe):
; -----------------------------------------------------------------------------------------------

DISP_RAHMEN:	lds	temp,dispmuster		; altes Muster laden
		push	temp			; ...und retten
; ----------------------------------------------
		lds	buffer,dispbreite	; Y-Breite
		sbrs	buffer,BIT0		; ...gerade?
		rjmp	disp_rvl		; ja...
		lsl	temp			; Muster um
		brcc	disp_rvlm		; ...eine Position
		inc	temp			; ...nach links schieben
DISP_RVLM:	sts	dispmuster,temp		; verschobenes Muster retten
;
DISP_RVL:	rcall	disp_vline		; linken Rand schreiben
		pop	temp			; altes Muter
		sts	dispmuster,temp		; ...wiederherstellen
;
		lds	temp,dispx		; X-Start
		push	temp			; ...retten
		lds	buffer,dispbreite	; ...und
		add	temp,buffer		; ...neuen X-Start
		sts	dispx,temp		; ...setzen
		rcall	disp_vline		; rechten Rand schreiben
		pop	temp			; alten X-Start
		sts	dispx,temp		; ...restaurieren
		rcall	disp_hline		; unteren Rand schreiben
;
		lds	temp,dispy		; Y-Start
		push	temp			; ...retten
		lds	buffer,disphoehe	; ...und
		add	temp,buffer		; ...neuen Y-Start
		dec	temp			; ...(Randkorrektur)
		sts	dispy,temp		; ...setzen
		rcall	disp_hline		; rechten Rand schreiben
; ----------------------------------------------
		pop	temp			; alten Y-Start
		sts	dispy,temp		; ...restaurieren
		ret



; -----------------------------------------------------------------------------------------------
; Rechteck ab (dispx), (dispy) mit (dispbreite) und (disphoehe) und Muster in (temp) füllen:
; -----------------------------------------------------------------------------------------------

DISP_RECT:	sts	dispmuster,temp		; Muster speichern
		push	counter
		lds	counter,dispx		; X-Start
		push	counter			; ...retten
		lds	counter,dispbreite	; Breite laden
		tst	counter			; Breite = 0?
		breq	disp_rectend		; ja...
DISP_RECTLOOP:	rcall	disp_vline		; eine Spalte schreiben
		lds	temp,dispx		; nächste
		inc	temp			; ...Spalte
		sts	dispx,temp		; ...setzen
		dec	counter			; Alle Spalten aufgefüllt?
		brne	disp_rectloop		; nein...
DISP_RECTEND:	pop	counter			; alten X-Start
		sts	dispx,counter		; ...restaurieren
		pop	counter
; ----------------------------------------------
		ret
		


; -----------------------------------------------------------------------------------------------
; Punkt an (dispx), (dispy) setzen:
; -----------------------------------------------------------------------------------------------

DISP_DOT:	ldi	temp,1			; nur 1 Punkt
		sts	dispbreite,temp		; ...setzen
		ldi	temp,$ff		; volles
		sts	dispmuster,temp		; ...Muster setzen
		clr	temp			; keine
		sts	dispeor,temp		; ...Musterinvertierung
		sts	dispmaske,temp		; ...und alle Bits verwenden
		rjmp	disp_hline



; -----------------------------------------------------------------------------------------------
; Horizontale Linie in (dispmuster) ab (dispx), (dispy) schreiben:
; -----------------------------------------------------------------------------------------------

DISP_HLINE:	push	YH
		push	YL
		push	counter
		push	buffer
		push	temp
		push	akku0
		push	akku1
		push	akku2
; ----------------------------------------------
		lds	akku2,dispbreite	; gewünschte Bitanzahl laden
		tst	akku2			; Breite=0?
		breq	disp_hend		; ja, nichts zu tun...
;
		ldiw	Y,dispram		; Anfang des aktiven Bildspeichers
		lds	counter,dispy		; Y-Start laden
		andi	counter,0b11111000	; Page
		lsr	counter			; ...aus
		lsr	counter			; ...Y-Start
		lsr	counter			; ...erzeugen
		ldi	temp,128		; relative Bildspeicheradresse
		mul	counter,temp		; ...in (akku1..0) erzeugen
		add	YL,akku0		; absolute Bildspeicheradresse
		adc	YH,akku1		; ...erzeugen
		lds	temp,dispx		; X-Offset
		add	YL,temp			; ...zum
		clr	temp			; ...Zeilenanfang
		adc	YH,temp			; ...addieren
;
		lds	counter,dispy		; Y-Start laden
		andi	counter,0b00000111	; ...und Y-Rest in der Page erzeugen
		ldi	temp,0b00000001		; oberstes Bit
		mov	akku0,temp		; ...am Anfang setzen
DISP_HL1:	tst	counter			; Schieben nötig?
		breq	disp_hlm		; nein...
		lsl	akku0			; ...von LSB -> MSB verschieben
		dec	counter			; Position z‰hlen
		rjmp	disp_hl1
;
DISP_HLM:	mov	akku1,akku0		; zu erhaltende Bits auf 1
		com	akku1			; ...und zu löschendes Bit auf 0 setzen
		lds	temp,dispmuster		; zu setzende Bits laden
DISP_HL2:	ld	buffer,Y		; eine Spalte in der Page holen
		and	buffer,akku1		; ...und eine 0 einstanzen
		lsl	temp			; soll eine 1 eingestanzt werden?
		brcc	disp_hsave		; nein...
		inc	temp			; 1 ins LSB zurück schreiben
		or	buffer,akku0		; Bit auf 1 setzen
DISP_HSAVE:	st	Y+,buffer		; ...und neue Spalte in der Page speichern
		dec	akku2			; ganze Breite bearbeitet?
		brne	disp_hl2		; nein...
; ----------------------------------------------
DISP_HEND:	pop	akku2
		pop	akku1
		pop	akku0
		pop	temp
		pop	buffer
		pop	counter
		pop	YL
		pop	YH
		ret



; -----------------------------------------------------------------------------------------------
; Vertikales Linie in (dispmuster) ab (dispx), (dispy) schreiben:
; -----------------------------------------------------------------------------------------------

DISP_VLINE:	push	YL
		push	YH
		push	ZH
		push	ZL
		push	counter
		push	buffer
		push	temp
		push	akku0
		push	akku1
		push	akku2
; ----------------------------------------------
; Oben bearbeiten:

		ldsw	Y,dispwp		; Anfang des aktiven Bildspeichers
		lds	akku2,disphoehe		; gewünschte Bitanzahl laden
		lds	counter,dispy		; Y-Start laden
		andi	counter,0b11111000	; oberste Page
		lsr	counter			; ...aus
		lsr	counter			; ...Y-Start
		lsr	counter			; ...erzeugen
		ldi	temp,128		; relative Bildspeicheradresse
		mul	counter,temp		; ...in (akku1..0) erzeugen
		add	YL,akku0		; absolute Bildspeicheradresse
		adc	YH,akku1		; ...erzeugen
		lds	temp,dispx		; X-Offset
		add	YL,temp			; ...zum
		clr	temp			; ...Zeilenanfang
		adc	YH,temp			; ...addieren
;
		clr	akku0			; Bitmaske ab LSB löschen
		clr	akku1			; Bitmaske ab MSB löschen
		lds	counter,dispy		; Y-Start laden
		andi	counter,0b00000111	; ...und Y-Start in der Page erzeugen
		breq	disp_vlm0		; kein oberer Teil nötig...
		clr	buffer
DISP_VLO1O:	sec				; 1-Bits (markieren nicht zu löschende Inhalte)
		ror	akku0			; ...vom LSB -> MSB einschieben
		dec	akku2			; Bitanzahl zählen
		inc	counter			; Bits in dieser Page zählen
		cpi	counter,8		; LSB-Maske fertig?
		brne	disp_vlo1o		; nein...
		com	akku0			; zu schützende Bits sind 1
		tst	akku2			; noch Bits vorhanden?
		brpl	disp_vol2		; alles passt in diese Page
DISP_VLO1U:	sec				; 1-Bits vom
		ror	akku1			; ...MSB -> LSB einschieben
		inc	akku2			; MSB-Maske fertig?
		brne	disp_vlo1u		; nein...
DISP_VOL2:	or	akku0,akku1		; alle 1-Bits sind geschützt
		mov	akku1,akku0		; nicht zu verändernde Bits
		com	akku1			; ...in der Page auf 0 setzen
		lds	temp,dispmuster		; zu setzende Bits laden
DISP_VLO3:	and	temp,akku1		; nicht zu verändernde Bits löschen
		ld	buffer,Y		; alte Bildspalte laden
		and	buffer,akku0		; ...und zu ver‰nderde Bits löschen
		or	buffer,temp		; neue Bildspalte erzeugen
		st	Y,buffer		; ...und speichern
		ldi	temp,low(128)		; adressiere
		add	YL,temp			; ...die
		clr	temp			; ...nächste
		adc	YH,temp			; ...Page
; ----------------------------------------------
; Mitte berarbeiten:

DISP_VLMITTE:	tst	akku2			; Noch Restbits vorhanden?
		breq	disp_vlend		; nein...
DISP_VLM0:	mov	counter,akku2		; ist noch mindestens
DISP_VLM1:	cpi	counter,8		; ...eine volle Page zu setzen?
		brcs	disp_vlunten		; nein...
		lds	temp,dispmuster		; zu setzende Bits laden
DISP_VLM2:	st	Y,temp			; ...und speichern
		ldi	temp,low(128)		; adressiere
		add	YL,temp			; ...die
		clr	temp			; ...n‰chste
		adc	YH,temp			; ...Page
		subi	counter,8		; eine Spalte abziehen, Underflow?
		breq	disp_vlend		; nichts mehr übrig...
		brpl	disp_vlm1		; nein...
; ----------------------------------------------
; Unten bearbeiten:

DISP_VLUNTEN:	lds	akku0,dispwp		; Startadresse
		lds	akku1,(dispwp+1)	; ...laden
		ldiw	Z,1024			; ...und
		add	akku0,ZL		; ...Endadresse
		adc	akku1,ZH		; ...berechnen
		cp	YH,akku1		; Endadresse (H) erreicht?
		brcs	disp_vlu0		; noch nicht...
		brne	disp_vlend		; ¸berschritten...
DISP_VLUT:	cp	YL,akku0		; Endadresse (L) erreicht?
		breq	disp_vlend		; ja...
		brcc	disp_vlend		; ¸berschritten...
DISP_VLU0:	clr	akku0			; Bitmaske lˆschen
DISP_VLU1:	sec				; 1-Bits vom
		rol	akku0			; ...LSB -> MSB einschieben
		dec	counter			; restliche Bits bearbeitet?
		brne	disp_vlu1		; nein...
		mov	akku1,akku0		; auszustanzende Bits
		com	akku1			; ...auf 0 setzen
		lds	temp,dispmuster		; zu setzende Bits laden
DISP_VLU2:	and	temp,akku0		; nicht zu verändernde Bits löschen
		ld	buffer,Y		; alte Bildspalte laden
		and	buffer,akku1		; ...und zu veränderde Bits löschen
		or	buffer,temp		; neue Bildspalte erzeugen
		st	Y,buffer		; ...und speichern
; ----------------------------------------------
DISP_VLEND:	pop	akku2
		pop	akku1
		pop	akku0
		pop	temp
		pop	buffer
		pop	counter
		pop	ZL
		pop	ZH
		pop	YH
		pop	YL
		ret



; -----------------------------------------------------------------------------------------------
; (akku0) mit Vorzeichen, deznmal, 2-stellig, rechts-ausgerichtet, ausgeben:
; -----------------------------------------------------------------------------------------------

DISP_DEZ2R:	push	akku0			; Zahlenwert retten
		push	akku1
		push	temp
		push	rest
		push	const
; ----------------------------------------------
		ldi	temp,SPACE		; für positive Zahlen
		tst	akku0			; negatives Vorzeichen nötig?
		brpl	disp_d2rvz		; nein...
		neg	akku0			; Wert positiv machen
		ldi	temp,'-'		; Vorzeichen laden
DISP_D2RVZ:	rcall	disp_out		; SPACE oder Minus ausgeben
		rcall	div8const10		; akku0 in Zehner und Einer aufspalten
		ldi	temp,SPACE		; Leerstelle bei Zehner=0
		tst	akku0			; Zehner = 0?
		breq	disp_d2rz		; ja...
		ldi	temp,$30		; Basis: ASCII 0
		add	temp,akku0		; Zehner-Ziffer erzeugen
DISP_D2RZ:	rcall	disp_out		; ...und ausgeben
		ldi	temp,$30		; Basis: ASCII 0
		add	temp,rest		; Einer-Ziffer erzeugen
		rcall	disp_out		; ...und ausgeben
; ----------------------------------------------
		pop	const
		pop	rest
		pop	temp
		pop	akku1
		pop	akku0			; Zahlenwert zurück holen
		ret



; -----------------------------------------------------------------------------------------------
; Neue Grafikzeile anfangen (dispy um Fonthöhe erhöhen):
; -----------------------------------------------------------------------------------------------

DISP_CRLF:	push	temp
		push	buffer
; ----------------------------------------------
		clr	temp			; Zeile ganz links
		sts	dispx,temp		; ...beginnen (CR)
;
		ldi	temp,6			; Höhe für Font 1
		lds	buffer,dispfont		; aktuell Font 1
		cpi	buffer,1		; ...eingestellt?
		breq	disp_nl			; ja...
		addi	temp,2			; Y-Korrektur für Font 2
DISP_NL:	lds	buffer,dispy		; Zeilenversatz
		add	buffer,temp		; ...addieren (LF)
		sts	dispy,buffer		; ...und retten
; ----------------------------------------------
		pop	buffer
		pop	temp
		ret



; -----------------------------------------------------------------------------------------------
; SPACE mit selektierten Font ausgeben (dispx um Fontbreite erhöhen):
; -----------------------------------------------------------------------------------------------

DISP_SPACE:	push	temp
; -------------
		ldi	temp,SPACE		; X-Position um Fontbreite
		rcall	disp_out		; ...nach rechts verschieben
; -------------
		pop	temp
		ret



; -----------------------------------------------------------------------------------------------
; ROM-String mit Begrenzung (counter) ab (Z+) in Bildspeicher schreiben:
; -----------------------------------------------------------------------------------------------

DISP_PRINTROM:	push	temp
		push	buffer
		push	akku0
		push	akku1
; ----------------------------------------------
		lds	akku0,dispx		; X-Start und Y-Start 
		lds	akku1,dispy		; ...für (CR) retten
DISP_PROMLOOP:	lpm	temp,Z+			; Zeichen laden
		tst	temp			; Ende gefunden?
		breq	disp_promend		; ja...
		tst	counter			; ohne Begrenzung ausgeben?
		brmi	disp_promtest		; ja...                                        
		dec	counter			; Ausgabe zählen, Begrenzung erreicht?
		brmi	disp_promend		; ja...

; ---------------------------------------------
; lokale X- und Y-Feinverschiebung testen:

DISP_PROMTEST:	mov	buffer,temp		; teste
		andi	buffer,0b11110000	; ...auf Feinverschiebung
		cpi	buffer,$f0		; X-Feinverschiebung gefunden?
		brne	disp_promyfein		; nein...
DISP_PROMXFEIN:	andi	temp,0b1111		; lower Anteil ist die Feinverschiebung
		sbrc	temp,BIT3		; Vorzeichen erweitern?
		ori	temp,0b11110000		; ja
		lds	buffer,dispx		; X-Korrektur
		add	buffer,temp		; ...nur für diese Zeile
		sts	dispx,buffer		; ...(bis CR) machen
		rjmp	disp_promloop
;
DISP_PROMYFEIN:	cpi	buffer,$e0		; Y-Feinverschiebung gefunden?
		brne	disp_promnorm		; nein...
		andi	temp,0b1111		; lower Anteil ist die Feinverschiebung
		sbrc	temp,BIT3		; Vorzeichen erweitern?
		ori	temp,0b11110000		; ja
		lds	buffer,dispy		; Y-Korrektur
		add	buffer,temp		; ...nur für diese Zeile
		sts	dispy,buffer		; ...(bis LF) machen
		rjmp	disp_promloop

; ---------------------------------------------
; normale Zeichenpositionierung:

DISP_PROMNORM:	cpi	temp,CR			; auf X-Start setzen?
		brne	disp_prolf		; nein...
		sts	dispx,akku0		; wieder auf X-Start setzen
		rjmp	disp_promloop
;
DISP_PROLF:	cpi	temp,LF			; um Zeichenhöhe nach unten verschieben?
		brne	disp_promout		; nein...
		lds	temp,dispfont		; aktive Fontnummer laden
		ldi	buffer,6		; Y-Offset des 4x6-Fonts
		cpi	temp,1			; kleiner Font eingestellt?
		breq	disp_proysto		; ja...
		ldi	buffer,8		; Y-Offset des 6x8-Fonts
		cpi	temp,2			; Normalfont eingestellt?
		breq	disp_proysto		; ja...
		clr	buffer			; kein Font gefunden: Offset ist 0
DISP_PROYSTO:	mov	temp,akku1		; ursprüngliche Y-Position laden
		add	temp,buffer		; ...Y-Fontoffset addieren
		sts	dispy,temp		; ...setzen
		mov	akku1,temp		; ...und Y-Start neu setzen
		rjmp	disp_promloop

; ---------------------------------------------
; Normale Zeichenausgabe machen:

DISP_PROMOUT:	rcall	disp_out		; ASCII-Zeichen ausgeben
		rjmp	disp_promloop

; ----------------------------------------------
DISP_PROMEND:	pop	akku1
		pop	akku2
		pop	buffer
		pop	temp
		ret



; -----------------------------------------------------------------------------------------------
; RAM-String mit Begrenzung (counter) ab (Z+) in Bildspeicher schreiben::
; -----------------------------------------------------------------------------------------------

DISP_PRINTRAM:	push	const
		push	buffer
; ----------------------------------------------
		lds	const,dispx		; X-Start für (CR) retten
DISP_PRAMLOOP:	ld	temp,Z+			; Zeichen laden
		tst	temp			; Ende gefunden?
		breq	disp_pramend		; ja...
		tst	counter			; ohne Ausgabebegrenzung?
		brmi	disp_pramok		; ja...     
		dec	counter			; Ausgabe z‰hlen, Begrenzung erreicht?
		brmi	disp_pramend		; ja...
DISP_PRAMOK:	cpi	temp,CR			; auf X-Start setzen?
		brne	disp_pralf		; nein...
		sts	dispx,const		; wieder auf X-Start setzen
		rjmp	disp_pramloop
;
DISP_PRALF:	cpi	temp,LF			; um Zeichenhöhe nach unten verschieben?
		brne	disp_praout		; nein...
		push	counter			; Ausgabebegrenzung retten
		lds	temp,dispfont		; aktive Fontnummer laden
		ldi	counter,6		; Y-Offset des 4x6-Fonts
		cpi	temp,1			; kleiner Font eingestellt?
		breq	disp_praysto		; ja...
		ldi	counter,8		; Y-Offset des 6x8-Fonts
		cpi	temp,2			; Normalfont eingestellt?
		breq	disp_praysto		; ja...
		clr	counter			; kein Font gefunden: Offset ist 0
DISP_PRAYSTO:	lds	temp,dispy		; aktuelle Y-Position laden
		add	temp,counter		; ...und passenden Fontoffset addieren
		sts	dispy,temp		; ...und setzen
		pop	counter			; Ausgabebegrenzung zurück laden
		rjmp	disp_pramloop
;
DISP_PRAOUT:	rcall	disp_out		; ASCII-Zeichen ausgeben
		rjmp	disp_pramloop
; ----------------------------------------------
DISP_PRAMEND:	pop	buffer
		pop	const
		ret



; -----------------------------------------------------------------------------------------------
; Ein Byte in (temp) in HEX in Bildspeicher mit Endabstand ausgeben:
; -----------------------------------------------------------------------------------------------

DISP_HEXSP:	rcall	disp_hex		; HEX-Ausgabe machen
		rjmp	disp_space		; dispx und Fontbreite erhöhen



; -----------------------------------------------------------------------------------------------
; Ein Byte in (temp) in HEX in Bildspeicher ausgeben:
; -----------------------------------------------------------------------------------------------

DISP_HEX:	push	temp			; für lower Nibble speichern
; ----------------------------------------------
		swap	temp			; higher Nibble zuerst
		rcall	disp_nibble		; ...ausgeben
		pop	temp			; lower Nibble laden

; 4 Bit in <temp> als Hex-Nibble ausgeben:

DISP_NIBBLE:	andi	temp,$F			; untere 4 Bits maskieren
		cpi	temp,10			; kleiner als 10?
		brlt	disp_makeascii		; ja...
		addi	temp,39			; Offset für a..f addieren
DISP_MAKEASCII:	addi	temp,0b00110000		; in HEX-Ziffer wandeln

; -----------------------------------------------------------------------------------------------
; ASCII-Zeichen in (temp) ab (dispy) ggf über 2 Pages schreiben:	[8,69µs/, 278/ cyc]
; -----------------------------------------------------------------------------------------------

DISP_OUT:	push	counter
		push	buffer
		lds	buffer,dispshift	; originale Feinverschiebung
		push	buffer			; ...retten
		lds	buffer,disppage		; Ausgabepage
		push	buffer			; ...retten
; ----------------------------------------------
		lds	buffer,dispx		; X-Position
		push	buffer			; ...retten
		push	temp			; ASCII-Zeichen retten
;
		lds	buffer,dispfont		; aktuellen Font laden
		ldi	temp,0b11000000		; Fontmaske laden
		cpi	buffer,1		; mit F4x6 ausgeben?
		breq	disp_outfu		; ja...
		ldi	temp,0b00000000		; Fontmaske laden
		cpi	buffer,2		; mit F6x8 ausgeben?
		breq	disp_outfu		; ja...
DISP_OUTEND:	pop	temp			; Stack säubern
		pop	buffer
		pop	buffer
		pop	buffer
		pop	buffer
		pop	counter
		ret
; ----------------------------------------------
DISP_OUTFU:	lds	buffer,dispy		; Y-Koordinate laden
		lds	counter,dispshift	; Feinverschiebung
		add	buffer,counter		; ...berücksichtigen
		mov	counter,buffer		; Vorbereiten für Pagenummer
		lsr	counter			; ...und ganzzahlige
		lsr	counter			; ...Page-1 für unteren
		lsr	counter			; ...Anteil erzeugen
		andi	buffer,0b111		; Pagerest erzeugen
		breq	disp_outo		; nur eine Pageausgabe nötig...
		inc	counter			; untere Pagenummer korrigieren
; 
; ggf. unteren Fontanteil ausgeben:
;
DISP_OUTU:	sts	disppage,counter	; Ausgabepage setzen
		push	temp			; initiale Bitmaske retten
		push	buffer			; Verschiebungsanzahl retten
		subi	buffer,8		; ...untere Bitmaske berechnen
		sts	dispshift,buffer	; ...und Fontverschiebung speichern
DISP_OUTUL:	tst	buffer			; Maske immer noch nach oben verschieben?
		breq	disp_outus		; nein...
		sec				; 1-Bit
		ror	temp			; ...ins MSB einfügen
		inc	buffer			; ...bis Zeilenrest =0
		rjmp	disp_outul
;
DISP_OUTUS:	sts	dispmaske,temp		; zu schützende Bits (=1) speichern
		pop	buffer			; Verschiebeanzahl holen
		pop	counter			; initiale Bitmaske holen
		pop	temp			; Kopie des ASCII-Zeichen
		push	temp			; ...holen
		call	disp_pageout		; Zeichen anzeigen
		mov	temp,counter		; initiale Bitmaske laden
		lds	counter,disppage	; Page
		dec	counter			; ...nach oben zurück setzen
;
; oberen Fontanteil ausgeben:
;
DISP_OUTO:	sts	dispshift,buffer	; Fontverschiebung speichern
		sts	disppage,counter	; Ausgabepage setzen
DISP_OUTOL:	tst	buffer			; Maske immer noch nach unten verschieben?
		breq	disp_outos		; nein...
		sec				; 1-Bit
		rol	temp			; ...ins LSB einfügen
		dec	buffer			; ...bis Zeilenrest =0
		rjmp	disp_outol
;
DISP_OUTOS:	sts	dispmaske,temp		; zu schützende Fontbits setzen
		pop	temp			; ASCII-Zeichen zurück laden
		pop	buffer			; X-Position zurück holen
		sts	dispx,buffer		; ...und setzen
		call	disp_pageout		; ...und oberen Fontteil anzeigen
; ----------------------------------------------
		pop	buffer			; Ausgabepage
		sts	disppage,buffer		; ...restaurieren
		pop	buffer			; originale Feinverschiebung
		sts	dispshift,buffer	; ...restaurieren
		pop	buffer
		pop	counter
		ret



; -----------------------------------------------------------------------------------------------
; ASCII-Zeichen (temp) in (disppage) ab (dispx) ausgeben:					[6,34µs, 203 cyc]
; -----------------------------------------------------------------------------------------------

DISP_PAGEOUT:	push	temp
		push	buffer
		push	counter
		push	rest
		push	const
		push	ZL
		push	ZH
		push	YL
		push	YH
		push	akku0
		push	akku1
		push	akku2
		push	akku3
; ----------------------------------------------
		ldsw	Y,dispwp		; aktueller Schreibbuffer
		lds	akku0,disppage		; Schreib-Page laden
		ldi	buffer,128		; Pagebreite laden
		mul	akku0,buffer		; (akku1..0) = Page-Offset in Bytes
		add	YL,akku0		; ...und absolte
		adc	YH,akku1		; ...Pageposition erzeugen
;
		lds	akku0,dispx		; Schreibspalte laden
		clr	akku1			; ...auf 16Bit erweitern
		add	YL,akku0		; ...und absolute
		adc	YH,akku1		; ...Schreibposition erzeugen
		lds	counter,dispfont	; aktuellen Font laden-
		cpi	counter,1		; kleiner Font?
		brne	disp_pagef2		; nein...
DISP_PAGEF1:	ldiw	Z,(2*F4x6)		; 4-Spalten-Font laden
		ldi	buffer,4		; Spalten pro Zeichen
		ldi	counter,4		; Fontbreite in Bytes pro Zeichen
		rjmp	disp_pagefout
;
		cpi	counter,2		; Standard-Font?
		brne	disp_pagerend		; nein...
DISP_PAGEF2:	ldiw	Z,(2*F6X8)		; 6-Spalten-Font laden
		ldi	buffer,6		; Spalten pro Zeichen
		ldi	counter,6		; Fontbreite in Bytes pro Zeichen
; -------------
; Fontausgabe:
; -------------
DISP_PAGEFOUT:	subi	temp,$20		; Tabelle fängt für 8Bit-Fonts mit 0 = SPACE an!
		brpl	disp_pageoutr		; alle Zeichen < $20
		ldi	temp,$5f		; ...wie DEL behandeln
DISP_PAGEOUTR:	mul	temp,counter		; Byte-Offset zum Fontanfang berechnen
		add	ZL,akku0		; ...und
		adc	ZH,akku1		; ...addieren
		lds	akku0,dispx		; aktuelle Schreib-Column erneut laden
		lds	akku1,dispshift		; Fontverschiebung laden
		lds	akku2,dispmaske		; zu schützende Bits laden
		mov	akku3,akku2		; inverse Maske
		com	akku3			; ...für Fontmuster erzeugen
		lds	const,dispeor		; EXOR-Maske für Spaltenbits
		mov	counter,buffer		; Anzahl der Spalten/Zeichen laden
;
DISP_PAGEOLOOP:	ld	buffer,Y		; altes Muster laden
		and	buffer,akku2		; zu setzende Pixel löschen
		lpm	temp,Z+			; 8 Spaltenpixel des Fontmusters laden
		mov	rest,akku1		; Verschiebungsanzahl laden
		tst	rest			; V-Shift nötig?
		breq	disp_pageoutst		; nein...
		brmi	disp_lcdvsup
DISP_LCDVSDN:	lsl	temp			; Font-Muster 1 Position nach unten schieben
		dec	rest			; Schiebeopoerationen fertig?
		brne	disp_lcdvsdn		; nein...
		rjmp	disp_pageoutst
;
DISP_LCDVSUP:	lsr	temp			; Font-Muster 1 Position nach oben schieben
		inc	rest			; Schiebeoperationen fertig?
		brne	disp_lcdvsup		; nein...
DISP_PAGEOUTST:	eor	temp,const		; Spaltenbits bearbeiten (pos/neg)
		and	temp,akku3		; zu setzende Pixel heraus lösen
		or	temp,buffer		; neues Ausgabemuster erzeugeh
		st	Y+,temp			; ...und im Display-RAM speichern
		inc	akku0			; Schreib-Column erhöhen
		dec	counter			; alle Column-Elemente ausgegeben?
		brne	disp_pageoloop		; nein...
DISP_PAGEREND:	sts	dispx,akku0		; nächste Column retten
; ----------------------------------------------
		pop	akku3
		pop	akku2
		pop	akku1
		pop	akku0
		pop	YH
		pop	YL
		pop	ZH
		pop	ZL
		pop	const
		pop	rest
		pop	counter
		pop	buffer
		pop	temp
		ret



; -----------------------------------------------------------------------------------------------
; Ganze Page im Display-RAM setzen:
; -----------------------------------------------------------------------------------------------

DISP_SETPAGE:	lds	buffer,disppage		; Pagenummer laden
DISP_SETZPAGEB:	ldi	counter,128		; Breite einer Page
		mul	buffer,counter		; Offset vom Display-Anfang in (akku1..0)
		ldsw	Z,dispwp		; akuelles Display-RAM
		add	ZL,akku0		; absolute Adresse
		adc	ZH,akku1		; ...der Page berechnen
;
		ldi	temp,$ff		; alle Bits ein
		rjmp	disp_ploop

; -----------------------------------------------------------------------------------------------
; Ganze Page im Display-RAM löschen:
; -----------------------------------------------------------------------------------------------

DISP_CLRPAGE:	lds	buffer,disppage		; Pagenummer laden
DISP_CLRPAGEB:	ldi	counter,128		; Breite einer Page
		mul	buffer,counter		; Offset vom Display-Anfang in (akku1..0)
		ldsw	Z,dispwp		; akuelles Display-RAM
		add	ZL,akku0		; absolute Adresse
		adc	ZH,akku1		; ...der Page berechnen
;
		clr	temp
		rjmp	disp_ploop

; -----------------------------------------------------------------------------------------------
; Ganze Page im Display-RAM löschen:
; -----------------------------------------------------------------------------------------------

DISP_FILLPAGE:	lds	buffer,disppage		; Pagenummer laden
DISP_FILLPAGEB:	ldi	counter,128		; Breite einer Page
		mul	buffer,counter		; Offset vom Display-Anfang in (akku1..0)
		ldsw	Z,dispwp		; akuelles Display-RAM
		add	ZL,akku0		; absolute Adresse
		adc	ZH,akku1		; ...der Page berechnen
;
DISP_PLOOP:	st	Z+,temp			; Pagespalte löschen
		dec	counter			; alles gelöscht?
		brne	disp_ploop		; nein...
		ret



; -----------------------------------------------------------------------------------------------
; Display-RAM setzen (dunkel):
; -----------------------------------------------------------------------------------------------

DISP_SETRAM:	ldi	temp,$ff		; alle Pixel dunkel
		rjmp	disp_fillram

; -----------------------------------------------------------------------------------------------
; Display-RAM löschen (hell):
; -----------------------------------------------------------------------------------------------

DISP_CLRRAM:	clr	temp			; alle Pixel leuchten

; -----------------------------------------------------------------------------------------------
; Display-RAM mit Muster in (temp) füllen:				[160,6µs, 51397cyc]
; -----------------------------------------------------------------------------------------------

DISP_FILLRAM:	ldiw	Y,1024			; Ende des Display-Buffers
		ldsw	Z,dispwp		; Anfang des Display-Buffers
DISP_FLOOP:	st	Z+,temp			; RAM setzen
		sbiw	Y,1			; Ende erreicht	
		brne	disp_floop		; nein...
; -------------
		ret	



; -----------------------------------------------------------------------------------------------
; Font 2 (normal) mit Standardwerten einstellen:
; -----------------------------------------------------------------------------------------------

DISP_FONT2:	push	temp
; ----------------------------------------------
		ldi	temp,2			; Standard-Font
		rjmp	disp_setfont

; -----------------------------------------------------------------------------------------------
; Font 1 (klein) mit Standardwerten einstellen:
; -----------------------------------------------------------------------------------------------

DISP_FONT1:	push	temp
; ----------------------------------------------
		ldi	temp,1			; kleinen Font
DISP_SETFONT:	sts	dispfont,temp		; ...laden
		clr	temp			; Font nicht
		sts	dispshift,temp		; ...nach unten verschieben
		sts	dispeor,temp		; ...und Font positiv anzeigen
		sts	dispmaske,temp		; ...alle Bits ver‰ndern
; ----------------------------------------------
		pop	temp
		ret



; -----------------------------------------------------------------------------------------------
; LCD-Page aus (buffer) setzen:
; -----------------------------------------------------------------------------------------------

DISP_PAGESET:	andi	buffer,0b111		; Page-Adresse auf 0..7 begrenzen
		ldi	temp,0b10110000		; Page-Befehl laden
		or	temp,buffer		; ...zusammen fügen
		rjmp	disp_cmd		; ...und Befehl ausgeben



; -----------------------------------------------------------------------------------------------
; LCD-Column aus (buffer) setzen:
; -----------------------------------------------------------------------------------------------

DISP_COLUMNSET:	mov	temp,buffer		; Column-Adresse laden
		swap	temp			; ...und MSB
		andi	temp,0b111		; ...heraus lösen
		ori	temp,0b00010000		; Befehl ergänzen
		rcall	disp_cmd		; ...und Column (H) setzen
;
		mov	temp,buffer		; Column-Adresse laden
		andi	temp,0b1111		; ...Column (L) heraus lösen und setzen

; -----------------------------------------------------------------------------------------------
; (temp) seriell ins Display-Kommandoregister schreiben:			[2,99µs, 94 cyc]
; -----------------------------------------------------------------------------------------------

DISP_CMD:	push	counter
; ----------------------------------------------
		cbi	VPORT3_OUT,LCD_A0	; Commando-Register setzen
		rjmp	disp_wstart

; -----------------------------------------------------------------------------------------------
; (temp) seriell ins Display-Datenregister schreiben:				[2,84µs, 91 cyc]
; -----------------------------------------------------------------------------------------------

DISP_WRITE:	push	counter
; ----------------------------------------------
		sbi	VPORT3_OUT,LCD_A0	; Daten-Register setzen
DISP_WSTART:	mov	counter,temp		; auszugebendes Byte retten
		pushsr				; ...weil hier (temp) benutzt wird
; -------------
		mov	temp,counter		; Ausgabe-Byte zurück holen
		cli				;;	Exklusiv
		cbi	VPORT3_OUT,LCD_CS	;;	Display selektieren
		ldi	counter,8		;;	8Bit ausgeben
DISP_WRLOOP:	cbi	VPORT3_OUT,LCD_SCL	;;	(t(SHW)>50ns) Takt in Ruheposition
		lsl	temp			;;	Daten ins MSB schieben
		brcs	disp_wr1		;;	1 gefunden...
DISP_WR0:	cbi	VPORT3_OUT,LCD_SI	;;	--> 0 ausgeben
		sbi	VPORT3_OUT,LCD_SCL	;;	aktive Taktflanke erzeugen
		dec	counter			;;	alle Bits gesendet?
		brne	disp_wrloop		;;	nein...
		rjmp	disp_wrend		;;
;
DISP_WR1: 	sbi	VPORT3_OUT,LCD_SI	;;	--> 1 ausgeben
		sbi	VPORT3_OUT,LCD_SCL	;;	(t(SDS)>20ns) aktive Taktflanke erzeugen
		dec	counter			;;	alle Bits gesendet?
		brne	disp_wrloop		;;	nein...
DISP_WREND:	sbi	VPORT3_OUT,LCD_CS	;;	Display deselektieren
; ----------------------------------------------
		popsr				;;	Interrupt wie zuvor
		pop	counter
		ret



; -----------------------------------------------------------------------------------------------
; LCD-Display initialisieren:
; -----------------------------------------------------------------------------------------------

DISP_INIT:	pushsr
; ----------------------------------------------
		sbi	VPORT3_OUT,LCD_RST	; Display-RST auf H legen
		rcall	dly100ms		; ...und etwas warten
		cbi	VPORT3_OUT,LCD_RST	; RESET durchführen
		rcall	dly100ms		; ...und etwas warten
		sbi	VPORT3_OUT,LCD_RST	; Grundzustand "H" einstellen
;
		ldiw	Z,(2*DOGM128)		; Initialisierungsbytes bis $FF
		cli				;;	Exklusiv
DISP_INITLOOP:	lpm	temp,Z+			;;	ein Byte laden
		cpi	temp,$ff		;;	Ende gefunden?
		breq	disp_initend		;;	ja...
		rcall	disp_cmd		;;	Byte ausgeben
		rjmp	disp_initloop		;;
;
DISP_INITEND:	ldiw	Z,dispram		;;	Adresse des Bildspeichers
		stsw	dispwp,Z		;;	...einstellen
		rcall	disp_font2		;;	Standard-Font 2 einstellen
		clr	temp			;;
		sts	disppage,temp		;;
		sts	dispbreite,temp		;;
		sts	disphoehe,temp		;;
		sts	dispx,temp		;;
		sts	dispy,temp		;;
		sts	dispeor,temp		;;
		sts	dispshift,temp		;;
		sts	dispmaske,temp		;;
; ----------------------------------------------
		popsr				;;	Interrupt wie zuvor
		ret



; -----------------------------------------------------------------------------------------------
; Division: (akku0) Rest (rest) = (akku0) : 10				ca: [2,38µs, 76 cyc]
; -----------------------------------------------------------------------------------------------

DIV8CONST10:	ldi	temp,10
		mov	const,temp

; -----------------------------------------------------------------------------------------------
; Division: (akku0) Rest (rest) = (akku0) : (const) für (const) < $80
; -----------------------------------------------------------------------------------------------

DIV8:		push	counter			; Zähler frei machen
; ----------------------------------------------
		ldi	counter,8		; 8 Bit Division
		clr	rest			; Rest löschen
DIV8_LOOP:	lsl	akku0			; erzeuge
		rol	rest			; ...8 Bit Zwischenergebnis
		cp	rest,const		; 8 Bit Divisor enthalten?
		brcs	div8_next		; nein...
		sub	rest,const		; Divisor abziehen
		inc	akku0			; ...und Ergebnisbit setzen
DIV8_NEXT:	dec	counter			; alle 8 Bit bearbeitet?
		brne	div8_loop		; nein...
; ----------------------------------------------
		pop	counter			; User-Zähler zurück laden
		ret



; ----------------------------------------------
; LCD-Initialisierung:

DOGM128:	.db	$e2, $40		; RESET			/ Display Start-Address
		.db	$a0, $c8		; Display RAM-Address	/ Reverse Direction
		.db	$a6, $a2		; Display normal mode	/ LCD Bias 1/9
		.db	$2f, $f8		; Booster on		/ Set Booster ratio ...
		.db	$00, $27		; ...2x, 3x, 4x		/ Voltage Regulator set
		.db	$81, $17		; Set contrast...	/ ...to $17
		.db	$ac, $00		; Indicator...		/ ...off	
		.db	$af, $a4		; Display on		/ Display normal
		.db	$FF			; (Endemarke)



; -------------------------------------
; Font1 für kleine Schrift:

F4X6:		.db	  0,  0,  0,  0		; (SPACE)
		.db	  0, 23,  0,  0		; !
		.db	  3,  0,  3,  0		; "
		.db	 31, 10, 31,  0		; #
		.db	 20, 51, 10,  0		; $
		.db	 25,  4, 19,  0		; %
		.db	 10, 21, 26,  0		; &
		.db	  0,  3,  0,  0		; '

		.db	  0, 14, 17,  0		; (
		.db	  0, 17, 14,  0		; )
		.db	 10,  4, 10,  0		; *
		.db	  4, 14,  4,  0		; +
		.db	 16,  8,  0,  0		; ,
		.db	  4,  4,  4,  0		; -
		.db	  0, 16,  0,  0		; .
		.db	$18,$04,$03,  0		; /

		.db	$0e, 17,$0e,  0		; 0
		.db	 18, 31, 16,  0		; 1
		.db	$19,$15,$12,  0		; 2
		.db	$15,$15,$0a,  0		; 3
		.db	  7,  4, 30,  0		; 4
		.db	$17,$15,$09,  0		; 5
		.db	$0e,$15,$08,  0		; 6
		.db	$01,$1d,$03,  0		; 7

		.db	$0a,$15,$0a,  0		; 8
		.db	$02,$15,$0e,  0		; 9
		.db	  0, 10,  0,  0		; :
		.db	 16, 10,  0,  0		; ;
		.db	  4, 10, 17,  0		; <
		.db	 10, 10, 10,  0		; =
		.db	 17, 10,  4,  0		; >
		.db	  1, 21,  3,  0		; ?

		.db	 30, 45, 46,  0		; @
		.db	 30,  5, 30,  0		; A
		.db	 31, 21, 10,  0		; B
		.db	 14, 17, 17,  0		; C
		.db	 31, 17, 14,  0		; D
		.db	 31, 21, 21,  0		; E
		.db	 31,  5,  5,  0		; F
		.db	 14, 17, 29,  0		; G

		.db	 31,  4, 31,  0		; H
		.db	 17, 31, 17,  0		; I
		.db	 24, 16, 31,  0		; J
		.db	 31,  4, 27,  0		; K
		.db	 31, 16, 16,  0		; L
		.db	 31,  2, 31,  0		; M
		.db	 31,  1, 30,  0		; N
		.db	$1f,$11,$1f,  0		; O

		.db	 31,  5,  2,  0		; P
		.db	$0e,$19,$16,  0		; Q
		.db	 31,  5, 26,  0		; R
		.db	 18, 21,  9,  0		; S
		.db	  1, 31,  1,  0		; T
		.db	 31, 16, 31,  0		; U
		.db	$0f,$10,$0f,  0		; V
		.db	 31,  8, 31,  0		; W

		.db	 27,  4, 27,  0		; X
		.db	  3, 28,  3,  0		; Y
		.db	 25, 21, 19,  0		; Z
		.db	  0, 31, 17,  0		; [
		.db	  3,  4, 24,  0		; \
		.db	  0, 17, 31,  0		; ]
		.db	  2,  1,  2,  0		; ^
		.db	 16, 16, 16, 16		; _

		.db	  0,  1,  2,  0		; `
		.db	 24, 20, 28,  0		; a
		.db	 31, 20,  8,  0		; b
		.db	  8, 20, 20,  0		; c
		.db	  8, 20, 31,  0		; d
		.db	 12, 26, 20,  0		; e
		.db	  4, 30,  5,  0		; f
		.db	 36, 42, 30,  0		; g

		.db	 31,  4, 24,  0		; h
		.db	  0, 29,  0,  0		; i
		.db	 32, 29,  0,  0		; j
		.db	 31,  8, 20,  0		; k
		.db	  0, 15, 16,  0		; l
		.db	 28,  8, 28,  0		; m
		.db	 28,  4, 24,  0		; n
		.db	  8, 20,  8,  0		; o

		.db	 60, 20,  8,  0		; p
		.db	  8, 20, 60,  0		; q
		.db	  0, 28,  4,  0		; r
		.db	 16, 28,  4,  0		; s
		.db	  2, 15, 18,  0		; t
		.db	 12, 16, 28,  0		; u
		.db	 12, 16, 12,  0		; v
		.db	 28, 24, 28,  0		; w
	
		.db	 20,  8, 20,  0		; x
		.db	 36, 40, 28,  0		; y
		.db	 20, 28, 20,  0		; z
		.db	  4, 31, 17,  0		; {
		.db	  0, 31,  0,  0		; |
		.db	 17, 31,  4,  0		; }
		.db	  1,  1,  1,  0		; ~
		.db	$0a,$14,$0a,  0		; ($7f) Schachbrettmuster
; -------------
		.db	$1f,$1f,$1f, 0		; ($80) 3x6 Blockcursor
		.db	$1d,$0a,$1d, 0		; ($81) "Ä"
		.db	$1d,$14,$1d, 0		; ($82) "Ö"
		.db	$1d,$10,$1d, 0		; ($83) "Ü"
		.db	$19,$14,$1d, 0		; ($84) "ä"
		.db	$1d,$14,$1d, 0		; ($85) "ö"
		.db	$0d,$10,$1d, 0		; ($86) "ü"
		.db	$02,$05,$02, 0		; ($87)	"°" Gradzeichen
		.db	$3e,$15,$0a, 0		; ($88) "ß" Beta-Zeichen
		.db	$04,$15,$0e,$04		; ($89) Pfeil nach rechts
		.db	$04,$0e,$15,$04		; ($8a) Pfeil nach links
		.db	$02,$3f,$02, 0		; ($8b) Pfeil nach oben
		.db	$10,$3f,$10, 0		; ($8c)	Pfeil nach unten
		.db	$00,$0e,$04, 0		; ($8d) Dreieck nach rechts
		.db	$04,$0e,$00, 0		; ($8e) Dreieck nach links
		.db	$02,$03,$02, 0		; ($8f) Dreieck nach oben
		.db	$08,$18,$08, 0		; ($90) Dreieck nach unten
		.db	$e0,$a0,$e0, 0		; ($91) offenes Quadrat
		.db	$e0,$e0,$e0, 0		; ($92) gefülltes Quadrat

; -------------
; Nur Font1:

		.db	  0,  0,  0,  4		; ($93) mittlerer Punkt auf letzter Spalte
		.db	  0,  4,  0,  0		; ($94) mittler Punkt auf 2. Spalte
		.db	  0,  0,  0, 14		; ($95) mittlerer Strich auf letzter Spalte
		.db	  0, 14,  0,  0		; ($96) mittler Strich auf 2. Spalte
		.db	$04,$04,$04,$04		; ($97) mittlerer horizontaler Strich



; ----------------------------------------------
; Dies ist der Normal-Font2

F6X8:		.db	  0,  0,  0,  0,  0,  0		; (Space)
		.db	  0,  0, 95,  0,  0,  0		; !
		.db	  0,  3,  0,  3,  0,  0		; "
		.db	 20,127, 20,127, 20,  0		; #
		.db	 36, 42,127, 42, 18,  0		; $
		.db	 35, 19,  8,100, 98,  0		; %
		.db	 54, 73, 85, 34, 80,  0		; &
		.db	  0,  5,  3,  0,  0,  0		; '

		.db	  0, 28, 34, 65,  0,  0		; (
		.db	  0, 65, 34, 28,  0,  0		; )
		.db	 20,  8, 62,  8, 20,  0		; *
		.db	  8,  8, 62,  8,  8,  0		; +
		.db	  0, 80, 48,  0,  0,  0		; ,
		.db	  8,  8,  8,  8,  8,  0		; -
		.db	  0, 96, 96,  0,  0,  0		; .
		.db	 32, 16,  8,  4,  2,  0		; /

		.db	 62, 81, 73, 69, 62,  0		; 0
		.db	  0, 66,127, 64,  0,  0		; 1
		.db	 98, 81, 73, 73, 70,  0		; 2
		.db	 34, 65, 73, 73, 54,  0		; 3
		.db	 24, 20, 18,127, 16,  0		; 4
		.db	 39, 69, 69, 69, 57,  0		; 5
		.db	 60, 74, 73, 73, 48,  0		; 6
		.db	  1, 97, 25,  5,  3,  0		; 7

		.db	 54, 73, 73, 73, 54,  0		; 8
		.db	  6, 73, 73, 41, 30,  0		; 9
		.db	  0, 54, 54,  0,  0,  0		; :
		.db	  0, 86, 54,  0,  0,  0		; ;
		.db	  8, 20, 34, 65,  0,  0		; <
		.db	 20, 20, 20, 20, 20,  0		; =
		.db	  0, 65, 34, 20,  8,  0		; >
		.db	  2,  1, 81,  9,  6,  0		; ?

		.db	 62, 65, 93, 85, 78,  0		; @
		.db	126,  9,  9,  9,126,  0		; A
		.db	127, 73, 73, 73, 54,  0		; B
		.db	 62, 65, 65, 65, 34,  0		; C
		.db	127, 65, 65, 34, 28,  0		; D
		.db	127, 73, 73, 73, 65,  0		; E
		.db	127,  9,  9,  9,  1,  0		; F
		.db	 62, 65, 73, 73, 58,  0		; G

		.db	127,  8,  8,  8,127,  0		; H
		.db	  0, 65,127, 65,  0,  0		; I
		.db	 32, 64, 65, 63,  1,  0		; J
		.db	127,  8, 20, 34, 65,  0		; K
		.db	127, 64, 64, 64, 64,  0		; L
		.db	127,  2, 12,  2,127,  0		; M
		.db	127,  4,  8, 16,127,  0		; N
		.db	 62, 65, 65, 65, 62,  0		; O

		.db	127,  9,  9,  9,  6,  0		; P
		.db	 62, 65, 81, 33, 94,  0		; Q
		.db	127,  9, 25, 41, 70,  0		; R
		.db	 38, 73, 73, 73, 50,  0		; S
		.db	  1,  1,127,  1,  1,  0		; T
		.db	 63, 64, 64, 64, 63,  0		; U
		.db	 31, 32, 64, 32, 31,  0		; V
		.db	 63, 64, 56, 64, 63,  0		; W

		.db	 99, 20,  8, 20, 99,  0		; X
		.db	  3,  4,120,  4,  3,  0		; Y
		.db	 97, 81, 73, 69, 67,  0		; Z
		.db	  0,127, 65, 65,  0,  0		; [
		.db	  3,  4,  8, 16, 96,  0		; \
		.db	  0, 65, 65,127,  0,  0		; ]
		.db	  4,  2,  1,  2,  4,  0		; ^
		.db	 64, 64, 64, 64, 64,  0		; _

		.db	  0,  1,  2,  4,  0,  0		; `
		.db	 32, 84, 84, 84,120,  0		; a
		.db	127, 72, 68, 68, 56,  0		; b
		.db	 56, 68, 68, 68, 32,  0		; c
		.db	 56, 68, 68, 72,127,  0		; d
		.db	 56, 84, 84, 84, 24,  0		; e
		.db	  0,  8,126,  9,  2,  0		; f
		.db	 24,164,164,164,124,  0		; g

		.db	127,  8,  4,  4,120,  0		; h
		.db	  0, 68,125, 64,  0,  0		; i
		.db	 32, 64, 64, 61,  0,  0		; j
		.db	127, 16, 40, 68,  0,  0		; k
		.db	  0, 65,127, 64,  0,  0		; l
		.db	124,  4, 24,  4,120,  0		; m
		.db	124,  8,  4,  4,120,  0		; n
		.db	 56, 68, 68, 68, 56,  0		; o

		.db	252, 40, 36, 36, 24,  0		; p
		.db	 24, 36, 36, 40,252,  0		; q
		.db	124,  8,  4,  4,  8,  0		; r
		.db	 72, 84, 84, 84, 32,  0		; s
		.db	  4, 63, 68, 64, 32,  0		; t
		.db	 60, 64, 64, 32,124,  0		; u
		.db	 28, 32, 64, 32, 28,  0		; v
		.db	 60, 64, 48, 64, 60,  0		; w

		.db	 68, 40, 16, 40, 68,  0		; x
		.db	 12, 80, 80, 80, 60,  0		; y
		.db	 68,100, 84, 76, 68,  0		; z
		.db	  0,  8, 54, 65,  0,  0		; {
		.db	  0,  0,127,  0,  0,  0		; |
		.db	  0, 65, 54,  8,  0,  0		; }
		.db	  3,  4,  2,  1,  6,  0		; ~
		.db	$aa,$55,$aa,$55,$aa,$55		; (DEL)

		.db	$7f,$7f,$7f,$7f,$7f,  0		; ($80) 5x7 Blockcursor
		.db	125, 10, 10, 10,125,  0		; ($81) Ä
		.db	 61, 66, 66, 66, 61,  0		; ($82) Ö
		.db	 62, 65, 64, 65, 62,  0		; ($83) Ü
		.db	 32, 85, 84, 85,120,  0		; ($84) ä
		.db	 56, 69, 68, 69, 56,  0		; ($85) ö
		.db	 60, 65, 64, 33,124,  0		; ($86) ü
		.db	$00,$06,$09,$09,$06,$00		; ($87) Grad-Zeichen (kleiner Kreis, oben)
		.db	$00,$fe,$49,$49,$36,$00		; ($88) Beta-Zeichen "ß"
		.db	  8,  8, 42, 28,  8,  0		; ($89) Pfeil nach rechts
		.db	  8, 28, 42,  8,  8,  0		; ($8a) Pfeil nach links
		.db	  4,  2,127,  2,  4,  0		; ($8b) Pfeil nach oben
		.db	 16, 32,127, 32, 16,  0		; ($8c) Pfeil nach unten
		.db	  0,  0, 62, 28,  8,  0		; ($8d) Dreieck nach rechts
		.db	  8, 28, 62,  0,  0,  0		; ($8e) Dreieck nach links
		.db	  4,  6,  7,  6,  4,  0		; ($8f) Dreieck nach oben
		.db	 16, 48,112, 48, 16,  0		; ($90) Dreieck nach unten
		.db	$14,$22,$00,$22,$14,$00		; ($91) offener Kreis
		.db	$1c,$3e,$3e,$3e,$1c,$00		; ($92)	gefüllter Kreis

; -------------
; Nur Font2:

		.db	$3e,$41,$5d,$55,$41,$3e		; ($93) Copyright-Zeichen
		.db	$80,$80,$80,$80,$80,$80		; ($94) unterste Zeile unter "underline" setzen
		.db	$24,$12,$24,$48,$24,$00		; ($95) "ungefähr gleich"-Zeichen
		.db	  8, 20, 34, 20,  8,  0		; ($96) Karo
		.db	 64,  0, 64,  0, 64,  0		; ($97) ...
		.db	  0,$22,$1c,$41,$3e,  0		; ($98) Sendefeld von links nach rechts
		.db	$3e,$41,$1c,$22,  0,  0		; ($99) Sendefeld nach rechts nach links	
		.db	$2a,$55,$22,$41,$00,$00		; ($9a) Raster  -> Öffnen
		.db	$2a,$55,$2a,$55,$2a,$55		; ($9b) 3-Zeilen Schachbrett in Mitte
		.db	$00,$41,$22,$55,$2a,$55		; ($9c) Schließen -> Raster





.dseg
; ********************************************************************************************
; RAM-Bereich:
; ********************************************************************************************


; --------------------------------------
; Display-Parameter:

dispx:		.byte	1			; X-Adresse im Bildspeicher
dispy:		.byte	1			; Y-Adresse im Bildspeicher
dispbreite:	.byte	1			; X-Breite
disphoehe:	.byte	1			; Y-Höhe
disppage:	.byte	1			; Page (0..7) im Display
dispmuster:	.byte	1			; Schreibmuster
dispeor:	.byte	1			; bitweise Invertierung des Schreibmusters
dispmaske:	.byte	1			; zu schützende Bits
dispshift:	.byte	1			; Y-Feinverschiebung
dispfont:	.byte	1			; Fontnummer

; ----------------------------------------------
; Anzeigebuffer:

dispwp:		.byte	2			; (L:H) Zeiger auf den Bildspeicheranfang
dispram:	.byte	(8*128)			; 1k-Byte Bildspeicher

